(ns clojure-demo.sitemap
  (:require [clojure.data.xml :as xml]
            [clojure-demo.routes :refer [routes]]

            ))


(defn pwa-sitemap [domain routes]
     (xml/emit-str
        {:tag :urlset
         :attrs {:xmlns "http://www.sitemaps.org/schemas/sitemap/0.9"}
         :content
         (mapv (fn [r]
                 {:tag :url
                  :attrs {}
                  :content [{:tag :loc :attrs {} :content [(str domain r)]}]}
                 ) routes)}))

(defn build-sitemap-urls [routes] 
  (->> routes
       (reduce (fn handle-sitemap [result [route route-map]]
                 (if (-> route-map :sitemap :vec)
                   (conj result ((-> route-map :sitemap :vec)))
                   (conj result route)
                   )) [])
       flatten))



(defn spit-pwa-sitemap2 
  "Generate the sitemap and outut to the provided path"
  [path domain urls]
  `(clojure.core/spit ~path (pwa-sitemap ~domain ~urls)))

(defmacro spit-pwa-sitemap-test
  "Generate the sitemap and outut to the provided path"
  [path domain urls]
  (clojure.core/spit
   path
   (pwa-sitemap
    domain
    urls)))

(defmacro spit-pwa-sitemap
  "Generate the sitemap and outut to the provided path"
  [path domain ]
  (clojure.core/spit
   path
   (pwa-sitemap
    domain
    (build-sitemap-urls routes))))

(comment
  (defn test-urls []
    ["a" "b" "c"])

  (spit-pwa-sitemap
   "resources/public/sitemap.xml"
   "https://clojure-demos.digitaloctave.com/"
   ["1" "2"])

  (spit-pwa-sitemap
   "resources/public/sitemap.xml"
   "https://clojure-demos.digitaloctave.com/"
   (test-urls))

  (macroexpand '(spit-pwa-sitemap
                 "resources/public/sitemap.xml"
                 "https://clojure-demos.digitaloctave.com/"
                 (test-urls))))
